#include <Serial.h>                                      // interrupt based I2C library
#include <Servo.h>                                     // library to drive up to 12 servos using timer1
#include "IOpins.h"                                    // defines which I/O pin is used for what function

// define constants here
#define startbyte 0x0F                                 // for serial communications each datapacket must start with this byte

// define global variables here
byte mode=0;                                           // mode=0: Serial / mode=3: Shutdown
int  lowbat=550;                                       // default low battery voltage is 5.5V
byte errorflag;                                        // non zero if bad data packet received
byte pwmfreq;                                          // value from 1-7
int lmspeed,rmspeed;                                   // left and right motor speeds -255 to +255
byte lmbrake,rmbrake;                                  // left and right brakes - non zero values enable brake
int lmcur,rmcur;                                       // left and right motor current
int lmenc,rmenc;                                       // left and right encoder values
int volts;                                             // battery voltage*10 (accurate to 1 decimal place)

byte servopin[6] = {7, 8, 12, 13, 5, 6};               // array stores IO pin for each servo
int servopos[6];                                       // array stores position data for up to 6 servos
Servo servo[6];                                        // create 6 servo objects as an array


void setup()
{
  // set timer 2 divisor to  256 for PWM frequency of 122.070312500 Hz
  TCCR2B = TCCR2B & B11111000 | B00000110;
  pwmfreq = 6;

  //all IO pins are input by default on powerup --------- configure motor control pins for output -------- pwm autoconfigures -----------
  pinMode(lmpwmpin,OUTPUT);                            // configure left  motor PWM       pin for output
  pinMode(lmdirpin,OUTPUT);                            // configure left  motor direction pin for output
  pinMode(lmbrkpin,OUTPUT);                            // configure left  motor brake     pin for output
  pinMode(rmpwmpin,OUTPUT);                            // configure right motor PWM       pin for output
  pinMode(rmdirpin,OUTPUT);                            // configure right motor direction pin for output
  pinMode(rmbrkpin,OUTPUT);                            // configure right motor brake     pin for output
  
  // open comm link at speed 38400
  Serial.begin(38400);
}


void loop()
{
  if (mode == 3)                                       // if battery voltage too low
  {
    Shutdown();                                        // Shutdown motors and servos
    return;
  }
  
  if (Serial.available()) command();                   // read serial command
}


void command()     
{
  byte b;                                                                      // byte from buffer
  int i;                                                                       // integer from buffer
  
  do                                                                           // check for start byte
  {
    b = Serial.read();                                                         // read a byte from the buffer
    if (b != startbyte) errorflag = errorflag | 1;                             // if byte does not equal startbyte or Master request incorrect number of bytes then generate error
  } while (errorflag > 0 && Serial.available() > 0);                           // if errorflag>0 then empty buffer of corrupt data
  
  if(errorflag > 0)                                                            // corrupt data received 
  {
    Shutdown();                                                                // shut down motors and servos
    return;                                                                    // wait for valid data packet
  }  
  //----------------------------------------------------------------------------- valid data packet received ------------------------------
  
  b = Serial.read();                                                           // read pwmfreq from the buffer
  if(b > 0 && b < 8)                                                               // if value is valid (1-7)  
  {
    pwmfreq = b;                                                                 // update pwmfreq
    TCCR2B = TCCR2B & B11111000 | pwmfreq;                                     // change timer 2 clock pre-scaler
  }
  else
  {
    errorflag = errorflag | 2;                                                 // incorrect pwmfreq given
  }
  
  i = Serial.read() * 256 + Serial.read();                                     // read integer from I²C buffer
  if(i > -256 && i < 256)
  {
    lmspeed = i;                                                                 // read new speed for   left  motor
  }
  else
  {
    errorflag = errorflag | 4;                                                 // incorrect motor speed given
  }
  lmbrake = Serial.read();                                                     // read new left  motor brake status
  
  i = Serial.read() * 256 + Serial.read();                                     // read integer from I²C buffer
  if(i > -256 && i < 256)
  {
    rmspeed=i;                                                                 // read new speed for   right motor
  }
  else
  {
    errorflag = errorflag | 4;                                                 // incorrect motor speed given
  }
  rmbrake = Serial.read();                                                     // read new right motor brake status
  
  if(errorflag & 4)                                                            // incorrect motor speed / shutdown motors 
  {
    lmspeed=0;                                                                 // set left  motor speed to 0
    rmspeed=0;                                                                 // set right motor speed to 0
  }
    
  for(byte j=0; j<6; j++)                                                      // read position information for 6 servos
  {
    i = Serial.read() * 256 + Serial.read();                                   // read integer from I²C buffer
    if (abs(i) > 2400) errorflag = errorflag | 8;                              // incorrect servo position given
    servopos[j] = i;                                                           // read new servo position -- 0 = no servo present
  }
  
  b = Serial.read();                                                             // read devibrate setting. this functionality has been dropped
  i = Serial.read() * 256 + Serial.read();                                     // read impact sensitivity setting. this functionality has been dropped
  
  i = Serial.read() * 256 + Serial.read();                                     // read integer from buffer
  if(i > 549 && i < 3001)
  {
    lowbat = i;                                                                // set low battery value (values higher than battery voltage will force a shutdown)
  }
  else
  {
    errorflag = errorflag | 32;                                                // incorrect lowbat given
  }
  
  b = Serial.read();                                                           // read byte from buffer (I²C address). this functionality has been dropped
  b = Serial.read();                                                           // read byte from buffer (I²C clock). this functionality has been dropped
  
  mode = 0;                                                                    // breaks out of Shutdown mode when I²C command is given
  Motors();                                                                    // update brake, speed and direction of motors
  Servos();                                                                    // update servo positions
}




